# @summary Performs zone creation and update operations. Internal class
#
# @param server_type
#   Controls whether this node is setup as a execution engine, or a feedmaster.
#
# @param service_user
#   Account which runs the groundplex engine.
# 
# @param service_group
#   Primary group of the account which runs the groundplex engine.
#
# @param slpropz_source
#   Source from which the config file is downloaded initially. Note that this
#   URL has an authentication component and currently can't be automatically
#   refreshed.
#
# @param array_msg_servers
#   List of SAP message servers which are allowed to connect to this snaplogic
#   instance. These are primarily used to create entries in /etc/services
class groundplex::config (
  Groundplex::ServerType $server_type                    = $groundplex::server_type,
  String $service_user                                   = $groundplex::service_user,
  String $service_group                                  = $groundplex::service_group,
  Stdlib::HTTPSUrl $slpropz_source                       = $groundplex::slpropz_source,
  Array[Groundplex::SapMessageServer] $array_msg_servers = $groundplex::array_msg_servers,
) {
  # Ensure the connection is allowed via the firewall
  # TODO - allow this to be disabled
  firewalld_custom_service { 'snaplogic_jcc_connections':
    ensure      => 'present',
    description => 'Snaplogic JCC HTTP & HTTPS Inbound Ports',
    ports       => [
      {
        'port'     => '8081',
        'protocol' => 'tcp',
      },
      {
        'port'     => '8090',
        'protocol' => 'tcp',
      },
    ],
  }
  firewalld_service { 'Public - Snaplogic - Allow JCC Connections':
    ensure  => 'present',
    service => 'snaplogic_jcc_connections',
    zone    => 'public',
  }

  case $server_type {
    'JCC': {
      # Nothing special is needed for JCC which doesn't apply to the feedmaster
    }
    'FeedMaster': {
      firewalld_custom_service { 'snaplogic_feedmaster_connections':
        ensure      => 'present',
        description => 'Snaplogic FeedMaster HTTP & HTTPS Inbound Ports',
        ports       => [
          {
            'port'     => '8084',
            'protocol' => 'tcp',
          },
          {
            'port'     => '8091',
            'protocol' => 'tcp',
          },
        ],
      }
      firewalld_service { 'Public - Snaplogic - Allow FeedMaster Connections':
        ensure  => 'present',
        service => 'snaplogic_feedmaster_connections',
        zone    => 'public',
      }
      firewalld_custom_service { 'snaplogic_feedmaster_broker':
        ensure      => 'present',
        description => 'Snaplogic FeedMaster Inbound Broker Ports',
        ports       => [
          {
            'port'     => '8089',
            'protocol' => 'tcp',
          },
        ],
      }
      firewalld_service { 'Public - Snaplogic - Allow FeedMaster Broker':
        ensure  => 'present',
        service => 'snaplogic_feedmaster_broker',
        zone    => 'public',
      }
    }
    # No-op default case
    default: {}
  }

  # Allow snaplogic to expand limits beyond default constraints
  limits::file { 'snaplogic_limits':
    sequence       => '10',
    header_comment => [
      'Prevents errors when higher pipeline loads execute.',
      'See https://docs-snaplogic.atlassian.net/wiki/spaces/SD/pages/1438367/Installing+a+Snaplex+on+Linux',
    ],
    entries        => {
      $service_user => [
        {
          'item'  => 'nproc',
          'type'  => 'soft',
          'value' => '8192',
        },
        {
          'item'  => 'nproc',
          'type'  => 'hard',
          'value' => '65536',
        },
        {
          'item'  => 'nofile',
          'type'  => 'soft',
          'value' => '8192',
        },
        {
          'item'  => 'nofile',
          'type'  => 'hard',
          'value' => '65536',
        },
      ],
    },
  }

  ## JCo IDoc Listener Message service entries
  # Standard SAP Gateway Entries
  $port_classes_sap = {
    'gw'       => {
      'comment'      => 'SAP System Gateway Port',
      'base_value'   => 3300,
      'name_pattern' => 'sapgw%02d',
    },
    'cs-abap'  => {
      'comment'      => 'SAP System Message Server ABAP Port',
      'base_value'   => 3600,
      'name_pattern' => 'sapms%02d',
      'prior_port'   => 'sapgw99',
    },
  }

  # Build message server lookup structures
  $ms_aliases_tmp = $array_msg_servers.map |$entry| {
    $number = $entry['number']
    $class = $entry['instance_class']
    $sid = $entry['sid']
    case $class {
      'cs-abap': { $name = "sapms${sid}" }
      'cs-java': { $name = "sapms${sid}j" }
      default: {}
    }
    if 'port_override' in $entry {
      $port = $entry['port_override']
    } else {
      $port = $port_classes_sap[$class]['base_value'] + $number
    }
    [
      "${number}_${sid}",
      {
        'name'  => $name,
        'port'  => $port,
        'class' => $class,
      },
    ]
  }
  $ms_aliases = Hash.new(flatten($ms_aliases_tmp))

  $port_classes_sap.each |$class, $data| {
    $comment = $data['comment']
    $base_value = $data['base_value']
    $name_pattern = $data['name_pattern']
    if 'port_multiplier' in $data {
      $multiplier = $data['port_multiplier']
    } else {
      $multiplier = 1
    }

    # Create the default port ranges
    Integer[0, 99].each |$port| {
      $port_id = sprintf('%02d', $port)
      $port_id_previous = sprintf('%02d', $port - 1)
      $port_number = $base_value + $port * $multiplier
      $port_name = sprintf($name_pattern, $port)

      # Construct the base argument
      $params_base = {
        ensure    => 'present',
        comment   => $comment,
        protocols => { 'tcp' => $port_number },
      }

      # Require either the previous port in this section or the last
      # port of the previous section
      if $port > 0 {
        $previous_name = sprintf($name_pattern, $port - 1)
        $params_require = { require => Etc_services[$previous_name] }
      } elsif 'prior_port' in $data {
        $previous_name = $data['prior_port']
        $params_require = { require => Etc_services[$previous_name] }
      } else {
        $params_require = {}
      }

      # Check the alias hash to see if any are relevant for this port
      $port_fixed_width = sprintf('%02d', $port)
      $port_regex = Regexp.new("^${port_fixed_width}_")
      $matching_aliases = $ms_aliases.filter |$entry| {
        $entry[0] =~ $port_regex and $entry[1]['class'] == $class
      }

      if length($matching_aliases) > 0 {
        $alias_list = $matching_aliases.map |$entry| {
          $entry[1]['name']
        }
        $params_aliases = { 'aliases' => $alias_list }
      } else {
        $params_aliases = {}
      }

      $params = $params_base + $params_require + $params_aliases
      etc_services { $port_name:
        * => $params,
      }
    }
  }

  # Retrieve the config file from our instance
  $target_file = '/opt/snaplogic/etc/groundplex.slpropz'
  exec { 'curl_slpropz':
    command => "/bin/curl '${slpropz_source}' -o '${target_file}'",
    cwd     => '/opt/snaplogic/etc',
    creates => $target_file,
    before  => File[$target_file],
  }

  file { $target_file:
    ensure => 'file',
    owner  => $service_user,
    group  => $service_group,
    mode   => '0600',
  }
}
